#!/usr/bin/env python

"An example login screen."

import WebStack.Generic
import md5

class LoginResource:

    "A resource providing a login screen."

    def __init__(self, authenticator, use_redirect=1):

        """
        Initialise the resource with an 'authenticator'. If the optional 'use_redirect'
        flag is set to 0, a confirmation screen is given instead of redirecting the user
        back to the original application.
        """

        self.authenticator = authenticator
        self.use_redirect = use_redirect

    def respond(self, trans):

        fields = trans.get_fields_from_body()
        redirect = ""

        if fields.has_key("redirect"):
            redirects = fields["redirect"]
            redirect = redirects[0]
            if self.authenticator.authenticate(trans):
                if self.use_redirect:
                    trans.set_header_value("Location", redirect)
                    trans.set_response_code(307)
                    return
                else:
                    self._show_success(trans, redirect)
                    return
        else:
            fields = trans.get_fields_from_path()
            if fields.has_key("redirect"):
                redirects = fields["redirect"]
                redirect = redirects[0]

        self._show_login(trans, redirect)

    def _show_login(self, trans, redirect):

        # When authentication fails or is yet to take place, show the login
        # screen.

        trans.set_content_type(WebStack.Generic.ContentType("text/html"))
        out = trans.get_response_stream()
        out.write("""
<html>
  <head>
    <title>Login Example</title>
  </head>
  <body>
    <h1>Login</h1>
    <form method="POST">
      <p>Username: <input name="username" type="text" size="12"/></p>
      <p>Password: <input name="password" type="text" size="12"/></p>
      <p><input name="login" type="submit" value="Login"/></p>
      <input name="redirect" type="hidden" value="%s"/>
    </form>
  </body>
</html>
""" % redirect)

    def _show_success(self, trans, redirect):

        # When authentication fails or is yet to take place, show the login
        # screen.

        trans.set_content_type(WebStack.Generic.ContentType("text/html"))
        out = trans.get_response_stream()
        out.write("""
<html>
  <head>
    <title>Login Example</title>
  </head>
  <body>
    <h1>Login Successful</h1>
    <p>Please proceed <a href="%s">to the application</a>.</p>
  </body>
</html>
""" % redirect)

    def _decode(self, url):

        "Decode the given 'url' for redirection purposes."

        return url.replace("%3f", "?").replace("%26", "&")

class LoginAuthenticator:

    def __init__(self, secret_key, credentials, cookie_name=None):

        """
        Initialise the authenticator with a 'secret_key', the authenticator's registry of
        'credentials' and an optional 'cookie_name'.

        The 'credentials' must be an object which supports tests of the form
        '(username, password) in credentials'.
        """

        self.secret_key = secret_key
        self.credentials = credentials
        self.cookie_name = cookie_name or "LoginAuthenticator"

    def authenticate(self, trans):

        # Process any supplied parameters.

        fields = trans.get_fields_from_body()

        if fields.has_key("username") and fields.has_key("password"):
            usernames, passwords = fields["username"], fields["password"]

            # Insist on only one username and password.

            if len(usernames) == 1 and len(passwords) == 1:
                username, password = usernames[0], passwords[0]

                # Check against the class's credentials.

                if (username, password) in self.credentials:

                    # Make a special cookie token.
                    # NOTE: This should be moved into a common library.

                    trans.set_cookie_value(
                        self.cookie_name,
                        username + ":" + md5.md5(username + self.secret_key).hexdigest()
                    )

                    return 1

        return 0

# vim: tabstop=4 expandtab shiftwidth=4
